from __future__ import unicode_literals
from dal import autocomplete
from teryt_tree.models import JednostkaAdministracyjna
from django.utils import six


class VoivodeshipAutocomplete(autocomplete.Select2QuerySetView):
    def get_queryset(self):
        qs = JednostkaAdministracyjna.objects.voivodeship().all()

        if self.q:
            qs = qs.filter(name__istartswith=self.q)
        return qs


class CountyAutocomplete(autocomplete.Select2QuerySetView):
    def get_queryset(self):
        qs = JednostkaAdministracyjna.objects.county().all()

        voivodeship = self.forwarded.get('voivodeship', None)

        if self.q:
            qs = qs.filter(name__istartswith=self.q)

        if voivodeship:
            return qs.filter(parent=voivodeship)
        return qs.none()


class CommunityAutocomplete(autocomplete.Select2QuerySetView):
    def get_result_label(self, result):
        return "%s (%s)" % (six.text_type(result), six.text_type(result.category))

    def get_queryset(self):
        qs = JednostkaAdministracyjna.objects.community().select_related('category').all()

        if self.q:
            qs = qs.filter(name__istartswith=self.q)

        county = self.forwarded.get('county', None)
        if county:
            return qs.filter(parent=county)
        return qs.none()



import django_filters
from django.shortcuts import get_object_or_404
from rest_framework import filters, viewsets
from teryt_tree.models import JednostkaAdministracyjna
from teryt_tree.rest_framework_ext.serializers import JednostkaAdministracyjnaSerializer


def custom_area_filter(queryset, value):
    if not value:
        return queryset
    return queryset.area(get_object_or_404(JednostkaAdministracyjna, pk=value))


class JednostkaAdministracyjnaFilter(filters.FilterSet):
    area = django_filters.CharFilter(action=custom_area_filter)

    class Meta:
        model = JednostkaAdministracyjna
        fields = ['name', 'category', 'category__level', 'area']


class JednostkaAdministracyjnaViewSet(viewsets.ModelViewSet):
    queryset = (JednostkaAdministracyjna.objects.
                select_related('category').
                prefetch_related('children').
                all())
    serializer_class = JednostkaAdministracyjnaSerializer
    filter_backends = (filters.DjangoFilterBackend,)
    filter_class = JednostkaAdministracyjnaFilter



# -*- coding: utf-8 -*-
import argparse
from django.core.management.base import BaseCommand, CommandError
from django.db import transaction
from django.utils.lru_cache import lru_cache

try:
    from lxml import etree
except ImportError:
    raise CommandError('Missing dependency. Please, install lxml>=2.3.3')

from teryt_tree.models import Category, JednostkaAdministracyjna


@lru_cache()
def get_genre(name, level):
    obj, _ = Category.objects.get_or_create(name=name, defaults={'level': level})
    return obj


class Command(BaseCommand):
    args = '<filename>'
    help = 'Creates a data in database base on TERYT.xml file.'
    PARENT_REDUCE = {2: 0,
                     4: 2,
                     7: 4}
    LEVEL_REDUCE = {2: 1,
                    4: 2,
                    7: 3}

    def add_arguments(self, parser):
        parser.add_argument('input', nargs='?', type=argparse.FileType('r'),
                            help="Input XML-file")
        parser.add_argument('--limit', default=0, type=int)

    @classmethod
    def to_object(cls, row, commit=True):
        data = {x.get('name').lower(): x.text for x in row}
        obj = JednostkaAdministracyjna()
        obj.active = True
        obj.id = "".join(data.get(x, '') or '' for x in ('woj', 'pow', 'gmi', 'rodz'))
        index = len(obj.pk)
        if len(obj.id) > 2:
            obj.parent = JednostkaAdministracyjna.objects.get(pk=obj.id[:cls.PARENT_REDUCE[index]])
        obj.name = data['nazwa'].title()
        obj.updated_on = data['stan_na']
        obj.category = get_genre(data['nazdod'], cls.LEVEL_REDUCE[index])
        return obj

    def handle(self, *args, **options):
        file = options['input'] or open(args[0], 'r')
        root = etree.parse(file)
        self.stdout.write(("Importing started. "
                           "This may take a few seconds. Please wait a moment.\n"))
        row_count = 0
        with transaction.atomic() and JednostkaAdministracyjna.objects.delay_mptt_updates():
            for row in root.iter('row'):
                Command.to_object(row).save()
                row_count += 1
                if row_count >= int(options['limit']) and int(options['limit']) != 0:
                    self.stdout.write("Limit reached, so break.")
                    break
        self.stdout.write("%s rows imported.\n" % row_count)



from rest_framework import serializers

from teryt_tree.models import JednostkaAdministracyjna
from teryt_tree.models import Category


class CategorySerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        model = Category
        fields = ('name', 'pk')


class JednostkaAdministracyjnaSerializer(serializers.HyperlinkedModelSerializer):
    teryt = serializers.CharField(source='pk', read_only=True)
    url = serializers.CharField(source='get_absolute_url', read_only=True)
    category = CategorySerializer()
    children = serializers.HyperlinkedRelatedField(
        many=True,
        read_only=True,
        view_name='jednostkaadministracyjna-detail'
    )

    class Meta:
        model = JednostkaAdministracyjna
        fields = ('url',
                  'parent',
                  'teryt',
                  'children',
                  'category',
                  'name',
                  'updated_on',
                  'active',
                  'level',)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

version = '0.11.0'

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    os.system('python setup.py bdist_wheel upload')
    sys.exit()

readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

setup(
    name='django-teryt-tree',
    version=version,
    description="Django-teryt-tree is a Django app that implements TERYT database as tree by django-mptt.",
    long_description=readme + '\n\n' + history,
    author='Adam Dobrawy',
    author_email='naczelnik@jawnosc.tk',
    url='https://github.com/ad-m/django-teryt-tree',
    packages=[
        'teryt_tree',
    ],
    include_package_data=True,
    install_requires=[
        'django-mptt',
        'django-model-utils',
        'django-autoslug',
        'factory-boy',
    ],
    download_url='https://github.com/ad-m/django-teryt-tree/tarball/%s' % (version),
    license="BSD",
    zip_safe=False,
    keywords='django-teryt-tree',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
)
